/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Application
    pisoFoam

Description
    Transient solver for incompressible flow.

    Turbulence modelling is generic, i.e. laminar, RAS or LES may be selected.

\*---------------------------------------------------------------------------*/

#include "fvCFD.H"
#include "singlePhaseTransportModel.H"
#include "turbulenceModel.H"
#include "pimpleControl.H"
#include "fvIOoptionList.H"
#include "IOporosityModelList.H"
#include "IOMRFZoneList.H"
#include "fixedFluxPressureFvPatchScalarField.H"
#include "DataEntry.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{
    #include "setRootCase.H"

    #include "createTime.H"
    #include "createMesh.H"
    #include "createFields.H"
    #include "createFvOptions.H"
    #include "initContinuityErrs.H"    
    #include "readTimeControls.H"    
    #include <sstream>
    
    pimpleControl pimple(mesh);

    // * * * * Read resistance, initial flux data & pulse properties * * * * * * * * * * * * * //

    Info<< "Reading pulse properties ..." << nl << endl;
    
    IOdictionary controlDictCopy
    (  IOobject  ("controlDict",runTime.system(),mesh,IOobject::MUST_READ,IOobject::NO_WRITE) );
    
    
    autoPtr<DataEntry<scalar> > nOutletsValue
    (  DataEntry<scalar>::New ("nOutlets", controlDictCopy) );
    int nOutlets = nOutletsValue().value(0);

    autoPtr<DataEntry<scalar> > startWriteTime
    (  DataEntry<scalar>::New   ("startWriteTime", controlDictCopy)  );
    scalar startWritingAt = startWriteTime().value(0);  
    
    autoPtr<DataEntry<scalar> > pulseDuration
    (  DataEntry<scalar>::New   ("pulseDuration", controlDictCopy)  );
  
        autoPtr<DataEntry<scalar> > timeStep
    (  DataEntry<scalar>::New  ("deltaT", controlDictCopy)  );

//        autoPtr<DataEntry<scalar> > writeInterval
//    (  DataEntry<scalar>::New  ("writeInterval", controlDictCopy)  );
    
  
  scalar pulseT = pulseDuration().value(0);
  scalar dt = timeStep().value(0);
//  scalar writeT = writeInterval().value(0);
  Info << "pulse duration: " << pulseT << nl;
  Info << "time step: " << dt << nl;
//  Info << "writeInterval: " << writeT << nl;
  
  //calculate nTimeStepsInPulse: 
  double ratio = pulseT / dt;
  double ratioRounded = ratio * 10000.0;
  int tmpInt = ratioRounded + 0.5;
  float tmpFloat = tmpInt;
  ratioRounded = tmpFloat / 10000.0;
  int ratioInt = ratioRounded;
  int nTimeStepsInPulse = ratioInt; 
  
  Info << "nTimeStepsInPulse: " << nTimeStepsInPulse << nl;

    std::ostringstream convertNum2Str;
    string resistanceString = "resistance";
    
    scalar resistanceArray[nOutlets]; 
    scalar initialFluxArray[nOutlets];
    scalar pressureScalingArray[nOutlets];
    int leftOrRightVentricleArray[nOutlets];

    for (int i=0; i<nOutlets; i++) 
    {
      std::ostringstream convertNum2Str;
      // read resistance values:
      convertNum2Str << i+1;
      string tmpString = "resistance"+convertNum2Str.str();
      autoPtr<DataEntry<scalar> > tmpResistanceValue
      (  DataEntry<scalar>::New (tmpString, controlDictCopy) );
      resistanceArray[i] = tmpResistanceValue().value(0);
            
      Info << "resistance @ outlet" + convertNum2Str.str() + ": " <<  resistanceArray[i] << nl;

      // read initial flux values:
      tmpString = "initialFlux"+convertNum2Str.str();
      autoPtr<DataEntry<scalar> > tmpInitialFluxValue
      (  DataEntry<scalar>::New (tmpString, controlDictCopy) );
      initialFluxArray[i] = tmpInitialFluxValue().value(0);

      // read capillaryPressureScaling:
      tmpString = "capillaryPressureScaling"+convertNum2Str.str();
      autoPtr<DataEntry<scalar> > tmpPressureScalingValue
      (  DataEntry<scalar>::New (tmpString, controlDictCopy) );
      pressureScalingArray[i] = tmpPressureScalingValue().value(0);  

      Info << "capillary Pressure Saling @ outlet" + convertNum2Str.str() + ": " <<  pressureScalingArray[i] << nl;

      // check if vessel supplies left or right ventricle: 1 -> left, 0 -> right
      tmpString = "leftOrRightVentricle"+convertNum2Str.str();
      autoPtr<DataEntry<scalar> > tmpLeftOrRightVentricleValue
      (  DataEntry<scalar>::New (tmpString, controlDictCopy) );
      leftOrRightVentricleArray[i] = tmpLeftOrRightVentricleValue().value(0);  

      if(leftOrRightVentricleArray[i] == 1)
      {
       Info << "outlet" + convertNum2Str.str() + " supplies left ventricle (boolean: " << leftOrRightVentricleArray[i] << "). " << nl;
      }
      else if(leftOrRightVentricleArray[i] == 0) 
      {
       Info << "outlet" + convertNum2Str.str() + " supplies right ventricle (boolean: " << leftOrRightVentricleArray[i] << "). " << nl;
      }
    }

    // capillary Pressure data vectors:
  Info << "Reading capillaryPressures" << endl;
//  Info << "before array" << endl;
  scalar capillaryPressureArray[nOutlets][nTimeStepsInPulse];
//  Info << "after array" << endl;
 for (int L=0; L<nOutlets; L++) 
 {
if(leftOrRightVentricleArray[L] == 1)
{
//Info << "Using LVCapillaryPressureArray for outlet" << L+1 << endl;
  scalar tCounter = 0;
   if((pressureScalingArray[L]-0) <= 0.143 /*equals ~1/7*/)
   {		
    Info << "capillaryPressureScaling for outlet" << L+1 << ": is between " << 0 << " & " << 0.143 << ", use LVCapillaryPressure125Dict" << endl;

    IOdictionary capillaryPressuresDict
    ( IOobject   ( "LVCapillaryPressure125Dict",runTime.system(),mesh,IOobject::MUST_READ,IOobject::NO_WRITE )  );
    autoPtr<DataEntry<scalar> > dataEntry (DataEntry<scalar>::New("capillaryPressure125",capillaryPressuresDict) );

    for (int i=0; i<nTimeStepsInPulse; i++)
    {
     tCounter = i*dt;
     capillaryPressureArray[L][i] = dataEntry().value(tCounter);
//     Info << "capillaryPressure @outlet" << L+1 << " for timestep " << tCounter << " is: " << capillaryPressureArray[L][i] << endl;
    }
   }
   else if((pressureScalingArray[L]-1*0.143) <= (0.143))
   {		
    Info << "capillaryPressureScaling for outlet" << L+1 << ": is between " << 1*0.143 << " & " << 2*0.143 << ", use LVCapillaryPressure115Dict" << endl; 

    IOdictionary capillaryPressuresDict
    ( IOobject   ( "LVCapillaryPressure115Dict",runTime.system(),mesh,IOobject::MUST_READ,IOobject::NO_WRITE )  );
    autoPtr<DataEntry<scalar> > dataEntry (DataEntry<scalar>::New("capillaryPressure115",capillaryPressuresDict) );

    for (int i=0; i<nTimeStepsInPulse; i++)
    {
     tCounter = i*dt;
     capillaryPressureArray[L][i] = dataEntry().value(tCounter);
//     Info << "capillaryPressure @outlet" << L+1 << " for timestep " << tCounter << " is: " << capillaryPressureArray[L][i] << endl;
    }
   }
   else if((pressureScalingArray[L]-2*0.143) <=  (0.143))
   {		
    Info << "capillaryPressureScaling for outlet" << L+1 << ": is between " << 2*0.143 << " & " << 3*0.143 << ", use LVCapillaryPressure105Dict" << endl;

    IOdictionary capillaryPressuresDict
    ( IOobject   ( "LVCapillaryPressure105Dict",runTime.system(),mesh,IOobject::MUST_READ,IOobject::NO_WRITE )  );
    autoPtr<DataEntry<scalar> > dataEntry (DataEntry<scalar>::New("capillaryPressure105",capillaryPressuresDict) );

    for (int i=0; i<nTimeStepsInPulse; i++)
    {
     tCounter = i*dt;
     capillaryPressureArray[L][i] = dataEntry().value(tCounter);
//     Info << "capillaryPressure @outlet" << L+1 << " for timestep " << tCounter << " is: " << capillaryPressureArray[L][i] << endl;
    }
   }
   else if((pressureScalingArray[L]-3*0.143) <= (0.143))
   {		
    Info << "capillaryPressureScaling for outlet" << L+1 << ": is between "<< 3*0.143 << " & " << 4*0.143 << ", use LVCapillaryPressure95Dict" << endl;

    IOdictionary capillaryPressuresDict
    ( IOobject   ( "LVCapillaryPressure95Dict",runTime.system(),mesh,IOobject::MUST_READ,IOobject::NO_WRITE )  );
    autoPtr<DataEntry<scalar> > dataEntry (DataEntry<scalar>::New("capillaryPressure95",capillaryPressuresDict) );

    for (int i=0; i<nTimeStepsInPulse; i++)
    {
     tCounter = i*dt;
     capillaryPressureArray[L][i] = dataEntry().value(tCounter);
 //    Info << "capillaryPressure @outlet" << L+1 << " for timestep " << tCounter << " is: " << capillaryPressureArray[L][i] << endl;
    }
   }
   else if((pressureScalingArray[L]-4*0.143) <= (0.143))
   {		
    Info << "capillaryPressureScaling for outlet" << L+1 << ": is between "<< 4*0.143 << " & " << 5*0.143 << ", use LVCapillaryPressure85Dict" << endl;

    IOdictionary capillaryPressuresDict
    ( IOobject   ( "LVCapillaryPressure85Dict",runTime.system(),mesh,IOobject::MUST_READ,IOobject::NO_WRITE )  );
    autoPtr<DataEntry<scalar> > dataEntry (DataEntry<scalar>::New("capillaryPressure85",capillaryPressuresDict) );

    for (int i=0; i<nTimeStepsInPulse; i++)
    {
     tCounter = i*dt;
     capillaryPressureArray[L][i] = dataEntry().value(tCounter);
//     Info << "capillaryPressure @outlet" << L+1 << " for timestep " << tCounter << " is: " << capillaryPressureArray[L][i] << endl;
    }
   }
   else if((pressureScalingArray[L]-5*0.143) <= (0.143))
   {		
    Info << "capillaryPressureScaling for outlet" << L+1 << ": is between "<< 5*0.143 << " & " << 6*0.143 << ", use LVCapillaryPressure75Dict" << endl;

    IOdictionary capillaryPressuresDict
    ( IOobject   ( "LVCapillaryPressure75Dict",runTime.system(),mesh,IOobject::MUST_READ,IOobject::NO_WRITE )  );
    autoPtr<DataEntry<scalar> > dataEntry (DataEntry<scalar>::New("capillaryPressure75",capillaryPressuresDict) );

    for (int i=0; i<nTimeStepsInPulse; i++)
    {
     tCounter = i*dt;
     capillaryPressureArray[L][i] = dataEntry().value(tCounter);
 //    Info << "capillaryPressure @outlet" << L+1 << " for timestep " << tCounter << " is: " << capillaryPressureArray[L][i] << endl;
    }
   }
   else if((pressureScalingArray[L]-6*0.143) <= (0.143))
   {		
    Info << "capillaryPressureScaling for outlet" << L+1 << ": is between "<< 6*0.143 << " & " << 7*0.143 << "~=" << 1 << ", use LVCapillaryPressure65Dict" << endl;

    IOdictionary capillaryPressuresDict
    ( IOobject   ( "LVCapillaryPressure65Dict",runTime.system(),mesh,IOobject::MUST_READ,IOobject::NO_WRITE )  );
    autoPtr<DataEntry<scalar> > dataEntry (DataEntry<scalar>::New("capillaryPressure65",capillaryPressuresDict) );

    for (int i=0; i<nTimeStepsInPulse; i++)
    {
     tCounter = i*dt;
     capillaryPressureArray[L][i] = dataEntry().value(tCounter);
//     Info << "capillaryPressure @outlet" << L+1 << " for timestep " << tCounter << " is: " << capillaryPressureArray[L][i] << endl;
    }
   }
}

else if(leftOrRightVentricleArray[L] == 0)
{
//Info << "Using RVCapillaryPressureArray for outlet" << L+1 << endl;
  scalar tCounter = 0;
   if((pressureScalingArray[L]-0) <= 0.143 /*equals ~1/7*/)
   {		
    Info << "capillaryPressureScaling for outlet" << L+1 << ": is between " << 0 << " & " << 0.143 << ", use RVCapillaryPressure125Dict" << endl;

    IOdictionary capillaryPressuresDict
    ( IOobject   ( "RVCapillaryPressure125Dict",runTime.system(),mesh,IOobject::MUST_READ,IOobject::NO_WRITE )  );
    autoPtr<DataEntry<scalar> > dataEntry (DataEntry<scalar>::New("capillaryPressure125",capillaryPressuresDict) );

    for (int i=0; i<nTimeStepsInPulse; i++)
    {
     tCounter = i*dt;
     capillaryPressureArray[L][i] = dataEntry().value(tCounter);
//     Info << "capillaryPressure @outlet" << L+1 << " for timestep " << tCounter << " is: " << capillaryPressureArray[L][i] << endl;
    }
   }
   else if((pressureScalingArray[L]-1*0.143) <= (0.143))
   {		
    Info << "capillaryPressureScaling for outlet" << L+1 << ": is between " << 1*0.143 << " & " << 2*0.143 << ", use RVCapillaryPressure115Dict" << endl; 

    IOdictionary capillaryPressuresDict
    ( IOobject   ( "RVCapillaryPressure115Dict",runTime.system(),mesh,IOobject::MUST_READ,IOobject::NO_WRITE )  );
    autoPtr<DataEntry<scalar> > dataEntry (DataEntry<scalar>::New("capillaryPressure115",capillaryPressuresDict) );

    for (int i=0; i<nTimeStepsInPulse; i++)
    {
     tCounter = i*dt;
     capillaryPressureArray[L][i] = dataEntry().value(tCounter);
//     Info << "capillaryPressure @outlet" << L+1 << " for timestep " << tCounter << " is: " << capillaryPressureArray[L][i] << endl;
    }
   }
   else if((pressureScalingArray[L]-2*0.143) <=  (0.143))
   {		
    Info << "capillaryPressureScaling for outlet" << L+1 << ": is between " << 2*0.143 << " & " << 3*0.143 << ", use RVCapillaryPressure105Dict" << endl;

    IOdictionary capillaryPressuresDict
    ( IOobject   ( "RVCapillaryPressure105Dict",runTime.system(),mesh,IOobject::MUST_READ,IOobject::NO_WRITE )  );
    autoPtr<DataEntry<scalar> > dataEntry (DataEntry<scalar>::New("capillaryPressure105",capillaryPressuresDict) );

    for (int i=0; i<nTimeStepsInPulse; i++)
    {
     tCounter = i*dt;
     capillaryPressureArray[L][i] = dataEntry().value(tCounter);
//     Info << "capillaryPressure @outlet" << L+1 << " for timestep " << tCounter << " is: " << capillaryPressureArray[L][i] << endl;
    }
   }
   else if((pressureScalingArray[L]-3*0.143) <= (0.143))
   {		
    Info << "capillaryPressureScaling for outlet" << L+1 << ": is between "<< 3*0.143 << " & " << 4*0.143 << ", use RVCapillaryPressure95Dict" << endl;

    IOdictionary capillaryPressuresDict
    ( IOobject   ( "RVCapillaryPressure95Dict",runTime.system(),mesh,IOobject::MUST_READ,IOobject::NO_WRITE )  );
    autoPtr<DataEntry<scalar> > dataEntry (DataEntry<scalar>::New("capillaryPressure95",capillaryPressuresDict) );

    for (int i=0; i<nTimeStepsInPulse; i++)
    {
     tCounter = i*dt;
     capillaryPressureArray[L][i] = dataEntry().value(tCounter);
 //    Info << "capillaryPressure @outlet" << L+1 << " for timestep " << tCounter << " is: " << capillaryPressureArray[L][i] << endl;
    }
   }
   else if((pressureScalingArray[L]-4*0.143) <= (0.143))
   {		
    Info << "capillaryPressureScaling for outlet" << L+1 << ": is between "<< 4*0.143 << " & " << 5*0.143 << ", use RVCapillaryPressure85Dict" << endl;

    IOdictionary capillaryPressuresDict
    ( IOobject   ( "RVCapillaryPressure85Dict",runTime.system(),mesh,IOobject::MUST_READ,IOobject::NO_WRITE )  );
    autoPtr<DataEntry<scalar> > dataEntry (DataEntry<scalar>::New("capillaryPressure85",capillaryPressuresDict) );

    for (int i=0; i<nTimeStepsInPulse; i++)
    {
     tCounter = i*dt;
     capillaryPressureArray[L][i] = dataEntry().value(tCounter);
//     Info << "capillaryPressure @outlet" << L+1 << " for timestep " << tCounter << " is: " << capillaryPressureArray[L][i] << endl;
    }
   }
   else if((pressureScalingArray[L]-5*0.143) <= (0.143))
   {		
    Info << "capillaryPressureScaling for outlet" << L+1 << ": is between "<< 5*0.143 << " & " << 6*0.143 << ", use RVCapillaryPressure75Dict" << endl;

    IOdictionary capillaryPressuresDict
    ( IOobject   ( "RVCapillaryPressure75Dict",runTime.system(),mesh,IOobject::MUST_READ,IOobject::NO_WRITE )  );
    autoPtr<DataEntry<scalar> > dataEntry (DataEntry<scalar>::New("capillaryPressure75",capillaryPressuresDict) );

    for (int i=0; i<nTimeStepsInPulse; i++)
    {
     tCounter = i*dt;
     capillaryPressureArray[L][i] = dataEntry().value(tCounter);
 //    Info << "capillaryPressure @outlet" << L+1 << " for timestep " << tCounter << " is: " << capillaryPressureArray[L][i] << endl;
    }
   }
   else if((pressureScalingArray[L]-6*0.143) <= (0.143))
   {		
    Info << "capillaryPressureScaling for outlet" << L+1 << ": is between "<< 6*0.143 << " & " << 7*0.143 << "~=" << 1 << ", use RVCapillaryPressure65Dict" << endl;

    IOdictionary capillaryPressuresDict
    ( IOobject   ( "RVCapillaryPressure65Dict",runTime.system(),mesh,IOobject::MUST_READ,IOobject::NO_WRITE )  );
    autoPtr<DataEntry<scalar> > dataEntry (DataEntry<scalar>::New("capillaryPressure65",capillaryPressuresDict) );

    for (int i=0; i<nTimeStepsInPulse; i++)
    {
     tCounter = i*dt;
     capillaryPressureArray[L][i] = dataEntry().value(tCounter);
//     Info << "capillaryPressure @outlet" << L+1 << " for timestep " << tCounter << " is: " << capillaryPressureArray[L][i] << endl;
    }
   }
}
 }




//scalar outFluxVector[nOutlets];
scalar outFluxArray[nOutlets];//[nTimeStepsInPulse];

  scalar inletPressureVector =0;
  scalar outletPressureVector[nOutlets];

  scalar localInflux;
  scalar totalInfluxSum;
  scalar totalOutflux;

  scalar totalInletPressureSum;
  scalar totalInletAreaSum;

//    scalar tCounter = 0;

  // * * * * Define and set variables * * * * * * * * * * * * * //
  
  word inletPatchName = "INLET"; 
  label inletPatch = mesh.boundaryMesh().findPatchID(inletPatchName);  

  word outletPatchNameArray[nOutlets];
  label outletPatchArray[nOutlets];
  for (int i=0; i<nOutlets; i++) 
  {
    std::ostringstream convertNum2Str;
    convertNum2Str << i+1;
    outletPatchNameArray[i] = "OUTLET"+convertNum2Str.str();
    outletPatchArray[i] = mesh.boundaryMesh().findPatchID(outletPatchNameArray[i]);
  }
  
  label thisProcNb = Pstream::myProcNo();
  int nProcessors = Pstream::nProcs();
  
  int timeCounter = 0;  // first time step is initial value -> starts with timeCounter=1, only runs from 0 - 9999
//  int fullCycleTimeCounter = 0; // can be larger than 9999, required for correct computation of deltaTimeCounter!
  scalar deltaTimeCounter = 0;
//  bool isBreak = false;
  
  scalar rho = 1060;

  // initialize flux arrays (arbitrary values):
//  scalar fluxArray[nOutlets][nTimeStepsInPulse];
  
  scalar newPressureValueArray[nOutlets];
//  scalar finalPressureValueArray[nOutlets][nTimeStepsInPulse];
//  scalar fluxesIntegral[nOutlets];
//  scalar finalPressureIntegral[nOutlets];

    surfaceScalarField phiTemp
  ( IOobject ("phiTemp",runTime.timeName(),mesh,IOobject::READ_IF_PRESENT,IOobject::NO_WRITE),
    linearInterpolate(U) & mesh.Sf() );

    
  
  // define fields & scalars that will be used for outflux etc. determination:

  scalarField totalInletPressureIntegral(nProcessors,0);
  scalarField totalInletArea(nProcessors,0);
  scalar localInletPressureIntegral;
  scalar localInletArea;
  
  int count = -1;    
 
    Info<< "\nStarting time loop\n" << endl;

    while (runTime.run())
    {
        #include "readTimeControls.H"
        #include "CourantNo.H"        
	#include "setDeltaT.H"
	
        runTime++;

        Info<< "Time = " << runTime.timeName() << nl << endl;


//************** START OF INSERTION FOR OUTLET PRESSURE CALCULATION AND ASSIGNMENT **************
    // * * * * check if sub-domain contains part of outlet  * * * * * * * * * * * // 
    
    scalar isOutletArray[nOutlets];
    for(int j=0; j<nOutlets;j++)
    {
      isOutletArray[j] = phi.boundaryField()[outletPatchArray[j]].size();   // size of boundary field
    }


//   timeCounter is assigned integer value according to runTime*nTimeStepsInPulse modulo nTimeStepsInPulse.
//	int integerMultipleOf_dt = static_cast<int> (1/dt);
	timeCounter = (static_cast<int> (runTime.value()/dt));//%nTimeStepsInPulse;
//	integerCounter = static_cast<int> (runTime.value())*integerMultipleOf_dt;

//    set deltaTimeCounter for computation of intermediate values of outletResistance values:
	if (runTime.value() - static_cast<scalar> (timeCounter)*dt < 1e-8)
	{
	
//	Info << "deltaTimeCounter smaller " << nl;
		deltaTimeCounter = dt;
	}
	else
	{
//	Info << "deltaTimeCounter larger " << nl;
		deltaTimeCounter = runTime.value() - static_cast<scalar> (timeCounter)*dt;
	}
	Info << "deltaTimeCounter: " << deltaTimeCounter << nl;

		    count++;

//************** END OF INSERTION FOR OUTLET PRESSURE CALCULATION AND ASSIGNMENT **************

        // Pressure-velocity PIMPLE corrector loop
	while (pimple.loop())
        {
		#include "UEqn.H"





            // --- Pressure Corrector Loop
		while (pimple.correct())
		{
			#include "pEqn.H"
		}

        
		if (pimple.turbCorr())
		{
 	       	turbulence->correct();
		}
	}
	
	// calculate Diffusion coefficient field:
    
	D = Dfactor / turbulence->nu();
	

    scalar t = runTime.value();

	if ( Pstream::parRun() )
	{

	  // get outfluxes and outlet pressures:
/*	  word outletPatchNameArray[nOutlets];
	  label outletPatchArray[nOutlets];
	  for (int i=0; i<nOutlets; i++) 
	  {
	    std::ostringstream convertNum2StrBla;
	    convertNum2StrBla << i+1;
	    outletPatchNameArray[i] = "OUTLET"+convertNum2StrBla.str();
	    outletPatchArray[i] = mesh.boundaryMesh().findPatchID(outletPatchNameArray[i]);
	  }
*/	  
	  scalar totalOutletArea[nOutlets];
	  scalar totalOutletPressureIntegral[nOutlets];
	  for(int outletCount=0; outletCount<nOutlets; outletCount++)
	  {
	    scalarField tmpOutletArea(nProcessors,0);
	    tmpOutletArea[thisProcNb] = sum(mesh.magSf().boundaryField()[outletPatchArray[outletCount]]);
	    reduce( tmpOutletArea, sumOp<scalarField>() );

	    scalarField tmpOutletPressureIntegral(nProcessors,0);
	    tmpOutletPressureIntegral[thisProcNb] = sum(mesh.magSf().boundaryField()[outletPatchArray[outletCount]]
		*p.boundaryField()[outletPatchArray[outletCount]]);
	    reduce( tmpOutletPressureIntegral, sumOp<scalarField>() );

	    scalarField tmpOutflux(nProcessors,0);
	    tmpOutflux[thisProcNb] = sum(phi.boundaryField()[outletPatchArray[outletCount]]);
	    reduce( tmpOutflux, sumOp<scalarField>() );
	    
	    // sum outfluxes, outlet areas and outlet pressure integrals of all processors:
	    totalOutletArea[outletCount] = 0;
	    totalOutletPressureIntegral[outletCount] = 0;
//		outFluxVector[outletCount] = 0;
	    outFluxArray[outletCount] = 0;
//	    outFluxArray[outletCount][timeCounter%nTimeStepsInPulse] = 0;
	    for (int co=0; co<nProcessors; co++)
	    { 
//		outFluxVector[outletCount] += tmpOutflux[co];
	      outFluxArray[outletCount] += tmpOutflux[co];
//	      outFluxArray[outletCount][timeCounter%nTimeStepsInPulse] += tmpOutflux[co];

	      totalOutletArea[outletCount] += tmpOutletArea[co];
	      totalOutletPressureIntegral[outletCount] += tmpOutletPressureIntegral[co];
	    }
	    outletPressureVector[outletCount] = totalOutletPressureIntegral[outletCount] / totalOutletArea[outletCount];
	  }		
	}
	else  // serial run
	{
		for(int outletCount=0; outletCount<nOutlets; outletCount++)
	      {
		outFluxArray[outletCount] = sum(phi.boundaryField()[outletPatchArray[outletCount]]);
	      }		
	}


//    int T = t * 10000000.0 + 0.5;    
//    if ( T % 100000 == 0 ) // write outfluxes and inlet pressure only if runtime is multiple of 0.01!
//    {

	// Determine influx, outfluxes using phi field (not phiTemp!):
	if ( Pstream::parRun() )
	{
	  
	      // get inlet pressure:
	      localInletPressureIntegral = sum(mesh.magSf().boundaryField()[inletPatch]
		*p.boundaryField()[inletPatch]);
	      localInletArea = sum(mesh.magSf().boundaryField()[inletPatch]);
	      scalarField totalInletPressureIntegral(nProcessors,0);
	      scalarField totalInletArea(nProcessors,0);
	      totalInletPressureIntegral[thisProcNb] = localInletPressureIntegral;
	      totalInletArea[thisProcNb] = localInletArea;
	      reduce( totalInletPressureIntegral, sumOp<scalarField>() );
	      reduce( totalInletArea, sumOp<scalarField>() );
	      // sum inlet pressures of all processors:	      
	      totalInletPressureSum = 0;
	      totalInletAreaSum = 0;
	      for (int co=0; co<nProcessors; co++)
	      { 
		totalInletPressureSum += totalInletPressureIntegral[co]; 
		totalInletAreaSum += totalInletArea[co]; 
	      }
	      
inletPressureVector = totalInletPressureSum / totalInletAreaSum;
	      Info << "inlet pressure: " << inletPressureVector << nl;	

	      // get Influx:
	      localInflux = sum(phi.boundaryField()[inletPatch]);
	      scalarField totalInflux(nProcessors,0);
	      totalInflux[thisProcNb] = localInflux;
	      reduce( totalInflux, sumOp<scalarField>() );
	      // sum inlet pressures of all processors:	      
	      totalInfluxSum = 0;
	      for (int co=0; co<nProcessors; co++)
	      { 
		totalInfluxSum += totalInflux[co]; 
	      }

	    Info << "Influx: " << (-1)*totalInfluxSum << nl;	    


	  totalOutflux = 0;	  
	  for(int outletCount=0; outletCount<nOutlets; outletCount++)
	  {
//	    Info << "outlet" << outletCount+1 << " flux:" << outFluxVector[outletCount] << nl;
//	    Info << "outlet" << outletCount+1 << " flux: " << outFluxArray[outletCount][timeCounter%nTimeStepsInPulse] << nl;
	    Info << "outlet" << outletCount+1 << " flux: " << outFluxArray[outletCount] << nl;
//	    Info << "outlet" << outletCount+1 << " flux of previous timeCounter:" << outFluxArray[outletCount][timeCounter%nTimeStepsInPulse-1] << nl;
	    Info << "outlet" << outletCount+1 << " pressure: " << outletPressureVector[outletCount] << nl;// from boundary field
	    Info << "outlet" << outletCount+1 << " pressure difference: " << inletPressureVector - outletPressureVector[outletCount] << nl;
	    Info << "outlet" << outletCount+1 << " capillaryPressure: " << capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse] +(capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse+1] - capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse]) / dt * deltaTimeCounter << nl;
//	    Info << "outlet" << outletCount+1 << " new pressure value array: " << newPressureValueArray[outletCount] << nl;
//	    Info << "outlet" << outletCount+1 << " final pressure value array: " << finalPressureValueArray[outletCount][timeCounter%nTimeStepsInPulse] << nl;	    	
//	    Info << "outlet" << outletCount+1 << " finalPressureIntegral: " << finalPressureIntegral[outletCount] << nl;	    
//	    Info << "outlet" << outletCount+1 << " fluxesIntegral: " << fluxesIntegral[outletCount] << nl;	
	    totalOutflux += outFluxArray[outletCount];                  
//	    totalOutflux += outFluxArray[outletCount][timeCounter%nTimeStepsInPulse];                  
	  }
  
	Info << "total Outflux: " << totalOutflux << nl;	  	  
	  
	  
	  
	}
	
	else  // serial run
	{
	      localInletPressureIntegral = sum(mesh.magSf().boundaryField()[inletPatch]
		*p.boundaryField()[inletPatch]);
	      localInletArea = sum(mesh.magSf().boundaryField()[inletPatch]);
      
inletPressureVector = localInletPressureIntegral / localInletArea;
	      Info << "inlet pressure: " << inletPressureVector << nl;	
	    Info << "Influx: " << (-1)*sum(phi.boundaryField()[inletPatch]) << nl;
	  
/*	  word outletPatchNameArray[nOutlets];
	  label outletPatchArray[nOutlets];
	  for (int i=0; i<nOutlets; i++) 
	  {
	    std::ostringstream convertNum2Str;
	    convertNum2Str << i+1;
	    outletPatchNameArray[i] = "OUTLET"+convertNum2Str.str();
	    outletPatchArray[i] = mesh.boundaryMesh().findPatchID(outletPatchNameArray[i]);
	  }
*/	  
	totalOutflux = 0;
	  for(int outletCount=0; outletCount<nOutlets; outletCount++)
	  {
/*	    outFluxVector[outletCount] = sum(phi.boundaryField()[outletPatchArray[outletCount]]);
	    Info << "outlet" << outletCount+1 << " flux:" << outFluxVector[outletCount] << nl;*/
	    outFluxArray[outletCount] = sum(phi.boundaryField()[outletPatchArray[outletCount]]);
//	    outFluxArray[outletCount][timeCounter%nTimeStepsInPulse] = sum(phi.boundaryField()[outletPatchArray[outletCount]]);
//	    Info << "outlet" << outletCount+1 << " flux: " << outFluxArray[outletCount][timeCounter%nTimeStepsInPulse] << nl;
	    Info << "outlet" << outletCount+1 << " flux: " << outFluxArray[outletCount] << nl;
outletPressureVector[outletCount] = (sum(mesh.magSf().boundaryField()[outletPatchArray[outletCount]]
		*p.boundaryField()[outletPatchArray[outletCount]]) ) / sum(mesh.magSf().boundaryField()[outletPatchArray[outletCount]]);
	    Info << "outlet" << outletCount+1 << " pressure: " << outletPressureVector[outletCount] << nl; // from boundary field
	    Info << "outlet" << outletCount+1 << " pressure difference: " << inletPressureVector - outletPressureVector[outletCount] << nl;
	    Info << "outlet" << outletCount+1 << " capillaryPressure: " << capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse]+(capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse+1] - capillaryPressureArray[outletCount][timeCounter%nTimeStepsInPulse]) / dt * deltaTimeCounter << nl;
//	    Info << "outlet" << outletCount+1 << " new pressure value array: " << newPressureValueArray[outletCount] << nl;
//	    Info << "outlet" << outletCount+1 << " final pressure value array: " << finalPressureValueArray[outletCount][timeCounter%nTimeStepsInPulse] << nl;	    	
//	    Info << "outlet" << outletCount+1 << " finalPressureIntegral: " << finalPressureIntegral[outletCount] << nl;	    
//	    Info << "outlet" << outletCount+1 << " fluxesIntegral: " << fluxesIntegral[outletCount] << nl;
//	    totalOutflux += outFluxArray[outletCount][timeCounter%nTimeStepsInPulse];              
	    totalOutflux += outFluxArray[outletCount];              
	  }
	  
	Info << "total Outflux: " << totalOutflux << nl;	

	}
    	Info << "timeCounter: " << timeCounter << nl;
//    }// write outfluxes and inlet pressure only if runtime is multiple of 0.01!

	
	
    if (runTime.value() >= startWritingAt )
    {
      runTime.write();
    }

        Info<< "ExecutionTime = " << runTime.elapsedCpuTime() << " s"
            << "  ClockTime = " << runTime.elapsedClockTime() << " s"
            << nl << endl;
    
    }
    

    Info<< "End\n" << endl;

    return 0;
}



// ************************************************************************* //
